@@ -2,6 +2,7 @@
 /*  $Header$
  *
  *  Copyright (c) 1987-2008 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 2008 Robert Ancell
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,7 +37,6 @@
 #include "config.h"
 #include "dsdefs.h"
 #include "functions.h"
-#include "lr_parser.h"
 #include "ce_parser.h"
 #include "mpmath.h"
 #include "display.h"
@@ -475,58 +475,12 @@ static XVars X;
 static void
 reset_display(void)
 {
-    struct exprm_state *e;
+    int *ans;
 
-    switch (v->syntax) {
-        case NPA:
-            v->ltr.noparens = 0;
-            MPstr_to_num("0", DEC, v->MPdisp_val);
-            display_set_number(v->MPdisp_val);
-            clear_undo_history();
-            break;
-
-        case EXPRS:
-            e = get_state();
-            MPstr_to_num("0", DEC, e->ans);
-            display_clear(FALSE);
-            display_set_number(e->ans);
-            break;
-        
-        default:
-            assert(0);
-    }
-}
-
-void
-ui_set_syntax_mode(enum syntax mode)
-{
-    GtkWidget *widget;
-
-    v->syntax = mode;
-    
-    reset_display();
-    
-    if (mode == NPA) {
-        ui_set_statusbar(_("Activated no operator precedence mode"), "");
-    } else {
-        ui_set_statusbar(
-            _("Activated expression mode with operator precedence"), "");
-    }
-    
-    /* Save the syntax mode */
-    set_resource(R_SYNTAX, Rsstr[v->syntax]);
-
-    /* ??? */
-    ui_set_mode(v->modetype);
-    
-    /* Update menu */
-    if (v->syntax == EXPRS) {
-        widget = GET_WIDGET("arithmetic_precedence_menu");
-        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(widget), TRUE);
-    } else {
-        widget = GET_WIDGET("ltr_precedence_menu");
-        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(widget), TRUE);
-    }
+    ans = display_get_answer(&v->display);
+    MPstr_to_num("0", DEC, ans);
+    display_clear(&v->display);
+    display_set_number(&v->display, ans);
 }
 
 
@@ -537,8 +491,6 @@ ui_set_accuracy(int accuracy)
     char text[MAXLINE];
     char *desc, *current, *tooltip;
     
-    v->accuracy = accuracy;
-    
     SNPRINTF(text, MAXLINE, _("_Other (%d) ..."), accuracy);
     widget = gtk_bin_get_child(GTK_BIN(GET_WIDGET("acc_item_other")));
     gtk_label_set_markup_with_mnemonic(GTK_LABEL(widget), text);
@@ -571,7 +523,8 @@ ui_set_accuracy(int accuracy)
     set_int_resource(R_ACCURACY, accuracy);
     
     ui_make_registers();
-    display_refresh(-1);
+    display_set_cursor(&v->display, -1);
+    display_refresh(&v->display);
     
     /* Hide the manual dialog */
     gtk_widget_hide(X->spframe);
@@ -603,17 +556,17 @@ ui_update_trig_mode()
     }
 
     gtk_label_set_markup(GTK_LABEL(GET_WIDGET("sine_label")),
-                         sine_labels[index]);
+                         _(sine_labels[index]));
     g_object_set_data(G_OBJECT(GET_WIDGET("calc_sine_button")), "calc_function",
                       GINT_TO_POINTER(sine_functions[index]));
 
     gtk_label_set_markup(GTK_LABEL(GET_WIDGET("cosine_label")),
-                         cosine_labels[index]);
+                         _(cosine_labels[index]));
     g_object_set_data(G_OBJECT(GET_WIDGET("calc_cosine_button")), "calc_function",
                       GINT_TO_POINTER(cosine_functions[index]));
 
     gtk_label_set_markup(GTK_LABEL(GET_WIDGET("tangent_label")),
-                         tangent_labels[index]);
+                         _(tangent_labels[index]));
     g_object_set_data(G_OBJECT(GET_WIDGET("calc_tangent_button")), "calc_function", 
                       GINT_TO_POINTER(tangent_functions[index]));
 }
@@ -660,7 +613,9 @@ ui_set_show_thousands_separator(gboolean visible)
     menu = GET_WIDGET("show_thousands_separator_menu");
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu), visible);
 
-    syntaxdep_show_display();
+    display_set_cursor(&v->display, -1);
+    display_refresh(&v->display);
+
     ui_make_registers();
 }
 
@@ -702,7 +657,9 @@ ui_set_show_trailing_zeroes(gboolean visible)
     v->show_zeroes = visible;
     set_boolean_resource(R_ZEROES, visible);
 
-    syntaxdep_show_display();
+    display_set_cursor(&v->display, -1);
+    display_refresh(&v->display);    
+
     ui_make_registers();
 
     menu = GET_WIDGET("show_trailing_zeroes_menu");
@@ -757,6 +714,64 @@ make_hostname()
 }
 
 
+gchar *
+ui_get_display(void)
+{
+    GtkTextIter start, end;
+    gtk_text_buffer_get_bounds(X->display_buffer, &start, &end);
+    return (gtk_text_buffer_get_text(X->display_buffer,
+                                     &start,
+                                     &end,
+                                     FALSE));
+}
+
+
+static int
+get_cursor(void)
+{
+    gint pos;
+    g_object_get(G_OBJECT(X->display_buffer), "cursor-position", &pos, NULL);
+    
+    /* Convert the last position to -1 */
+    if (pos == gtk_text_buffer_get_char_count(X->display_buffer)) {
+        return (-1);
+    } else {
+        return (pos);
+    }
+}
+
+
+static void
+set_bit_panel(void)
+{
+    int i;
+    const gchar *label;
+    guint64 value;
+    
+    if (!display_get_unsigned_integer(&v->display, &value))
+    {
+        gtk_widget_set_sensitive(X->bit_panel, FALSE);        
+        return;
+    }
+    gtk_widget_set_sensitive(X->bit_panel, TRUE);
+
+    for (i = 0; i < MAXBITS; i++) {
+        if (value & (1LL << (MAXBITS-i-1)))
+            label = " 1";
+        else
+            label = " 0";
+        gtk_label_set_text(GTK_LABEL(X->bits[i]), label);
+    }
+}
+
+
+static void do_button(int function, int arg)
+{
+    do_expression(function, arg, get_cursor());
+    set_bit_panel();
+}
+
+
 void
 ui_set_mode(enum mode_type mode)
 {
@@ -770,13 +785,13 @@ ui_set_mode(enum mode_type mode)
 
         ui_set_base(DEC);
         ui_set_numeric_mode(FIX);
-        ui_set_accuracy(DEFAULT_ACCURACY);
+        do_button(KEY_SET_ACCURACY, DEFAULT_ACCURACY);
         ui_set_show_thousands_separator(FALSE);
         ui_set_show_trailing_zeroes(FALSE);
         ui_make_registers();
 
         /* Reset display */
-        do_clear();
+        display_reset(&v->display);
         ui_set_statusbar("", "");
     }
     
@@ -880,68 +895,6 @@ ui_set_statusbar(const gchar *text, const gchar *imagename)
     gtk_statusbar_push(GTK_STATUSBAR(X->statusbar), 0, text); 
 }
 
-static void
-set_bit_panel(void)
-{
-    int bit_str_len, i, MP1[MP_SIZE], MP2[MP_SIZE];
-    int MP[MP_SIZE];
-    char bit_str[MAXLINE], label[MAXLINE];
-
-    switch (v->syntax) {
-        case NPA:
-            MPstr_to_num(v->display, v->base, MP1);
-            mpcmim(MP1, MP2);
-            if (mp_is_equal(MP1, MP2)) {
-                int toclear = (v->current == KEY_CLEAR_ENTRY)
-                              ? TRUE : FALSE;
-
-                make_fixed(bit_str, MAXLINE, MP1, BIN, MAXLINE, toclear);
-                bit_str_len = strlen(bit_str);
-                if (bit_str_len <= MAXBITS) {
-                    gtk_widget_set_sensitive(X->bit_panel, TRUE);
-
-                    for (i = 0; i < MAXBITS; i++) {
-                        if (i < bit_str_len) {
-                            SNPRINTF(label, MAXLINE, " %c", bit_str[bit_str_len-i-1]);
-                        } else {
-                            SNPRINTF(label, MAXLINE, " 0");
-                        }
-                        gtk_label_set_text(GTK_LABEL(X->bits[MAXBITS - i - 1]), label);
-                    }
-
-                    return;
-                }
-            }
-            gtk_widget_set_sensitive(X->bit_panel, FALSE);
-            break;
-
-        case EXPRS: 
-            if (display_is_usable_number(MP) || !is_integer(MP)) {
-                gtk_widget_set_sensitive(X->bit_panel, FALSE);
-                return;
-            }
-            make_fixed(bit_str, MAXLINE, MP, BIN, MAXLINE, FALSE);
-            bit_str_len = strlen(bit_str);
-            if (bit_str_len <= MAXBITS) {
-                gtk_widget_set_sensitive(X->bit_panel, TRUE);
-                
-                for (i = 0; i < MAXBITS; i++) {
-                    if (i < bit_str_len) {
-                        SNPRINTF(label, MAXLINE, " %c", bit_str[bit_str_len-i-1]);
-                    } else {
-                        SNPRINTF(label, MAXLINE, " 0");
-                    }
-                    gtk_label_set_text(GTK_LABEL(X->bits[MAXBITS - i - 1]), label);
-                }
-            } else {
-                gtk_widget_set_sensitive(X->bit_panel, FALSE);
-            }
-            break;
-
-        default:
-            assert(FALSE);
-    }
-}
 
 static gboolean
 redo_display(gpointer data)
@@ -975,10 +928,8 @@ ui_set_display(char *str, int cursor)
     if (str == NULL || str[0] == '\0') {
         str = " ";
     } else {
-        if (v->syntax == EXPRS || (v->syntax == NPA && v->ltr.noparens == 0)) {
-            localize_expression(localized, str, MAX_LOCALIZED, &cursor);
-            str = localized;
-        }
+        localize_expression(localized, str, MAX_LOCALIZED, &cursor);
+        str = localized;
     }
 
     gtk_text_buffer_set_text(X->display_buffer, str, -1);
@@ -1055,10 +1006,6 @@ ui_set_error_state(gboolean error)
                              !v->error); 
     gtk_widget_set_sensitive(GET_WIDGET("show_bitcalculating_menu"), !v->error);
     gtk_widget_set_sensitive(GET_WIDGET("show_registers_menu"), !v->error); 
-    gtk_widget_set_sensitive(GET_WIDGET("ltr_precedence_menu"), !v->error);
-
-    gtk_widget_set_sensitive(GET_WIDGET("arithmetic_precedence_menu"),
-                             !v->error); 
 
     gtk_widget_set_sensitive(GET_WIDGET("about_menu"), !v->error);
 }
@@ -1112,33 +1059,6 @@ ui_set_registers_visible(gboolean visible)
 }
 
 
-gchar *
-ui_get_display(void)
-{
-    GtkTextIter start, end;
-    gtk_text_buffer_get_bounds(X->display_buffer, &start, &end);
-    return (gtk_text_buffer_get_text(X->display_buffer,
-                                     &start,
-                                     &end,
-                                     FALSE));
-}
-
-
-static int
-get_cursor(void)
-{
-    gint pos;
-    g_object_get(G_OBJECT(X->display_buffer), "cursor-position", &pos, NULL);
-    
-    /* Convert the last position to -1 */
-    if (pos == gtk_text_buffer_get_char_count(X->display_buffer)) {
-        return (-1);
-    } else {
-        return (pos);
-    }
-}
-
-
 /*ARGSUSED*/
 static void
 about_cb(GtkWidget *widget)
@@ -1183,7 +1103,7 @@ about_cb(GtkWidget *widget)
             "authors", authors,
             "documenters", documenters,
             "translator_credits", translator_credits,
-            "logo-icon-name", "gnome-calculator",
+            "logo-icon-name", "accessories-calculator",
             NULL);
 }
 
@@ -1195,7 +1115,6 @@ cell_edited_cb(GtkCellRendererText *cell, const gchar *path_string,
     GtkTreeModel *model = (GtkTreeModel *) data;
     GtkTreePath *path = gtk_tree_path_new_from_string(path_string);
     GtkTreeIter iter;
-    gchar *old_text;
     gint *column;
 
     column = g_object_get_data(G_OBJECT(cell), "column");
@@ -1204,17 +1123,13 @@ cell_edited_cb(GtkCellRendererText *cell, const gchar *path_string,
 
     switch (GPOINTER_TO_INT(column)) {
         case COLUMN_VALUE:
-            gtk_tree_model_get(model, &iter, column, &old_text, -1);
-            g_free(old_text); 
             gtk_list_store_set(GTK_LIST_STORE(model), &iter, column,
-                           g_strdup(new_text), -1);
+                               g_strdup(new_text), -1);
             break;
 
         case COLUMN_DESCRIPTION:
-            gtk_tree_model_get(model, &iter, column, &old_text, -1);
-            g_free(old_text);
             gtk_list_store_set(GTK_LIST_STORE(model), &iter, column,
-                     g_strdup(new_text), -1);
+                               g_strdup(new_text), -1);
             break;
     }
  
@@ -1259,7 +1174,7 @@ aframe_response_cb(GtkWidget *dialog, gint response_id)
     if (response_id == GTK_RESPONSE_OK) {
         ch = (char *) gtk_entry_get_text(GTK_ENTRY(X->aframe_ch));
         mp_set_from_integer(ch[0], v->MPdisp_val);
-        display_set_number(v->MPdisp_val);
+        display_set_number(&v->display, v->MPdisp_val);
     }
     
     gtk_widget_hide(dialog);
@@ -1303,7 +1218,7 @@ void
 disp_cb(GtkWidget *widget)
 {
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-        do_numtype((enum num_type) g_object_get_data(G_OBJECT(widget), "numeric_mode"));
+        do_button(KEY_SET_NUMBERTYPE, (int)g_object_get_data(G_OBJECT(widget), "numeric_mode"));
 }
 
 
@@ -1316,44 +1231,7 @@ base_cb(GtkWidget *widget)
     base = (enum base_type) g_object_get_data(G_OBJECT(widget),
                                               "base_mode");
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
-        do_base(base);
-    }
-}
-
-
-static void do_button(int function, int arg)
-{
-    switch (v->syntax) {
-        case NPA:
-            v->current = buttons[function].id;
-
-            if (v->error) {
-                /* Must press a valid key first. */
-                if (v->current != KEY_CLEAR) {
-                    return;
-                }
-                ui_set_error_state(FALSE);
-            }
-    
-            if (v->ltr.noparens > 0) {
-                do_paren();
-                return;
-            }
-
-            buttons[function].func(arg);
-            set_bit_panel();
-            if (v->ltr.new_input && v->dtype == FIX) {
-                display_set_string(v->display);
-            }
-            break;
-
-        case EXPRS:
-            do_expression(function, arg, get_cursor());
-            set_bit_panel();
-            break;
-
-        default:
-            assert(0);
+        do_button(KEY_SET_BASE, base);
     }
 }
 
@@ -1616,8 +1494,8 @@ create_constants_model()
         gtk_list_store_set(model, &iter,
                            COLUMN_NUMBER, i,
                            COLUMN_EDITABLE, TRUE,
-                           COLUMN_VALUE, g_strdup(constant),
-                           COLUMN_DESCRIPTION, g_strdup(v->con_names[i]),
+                           COLUMN_VALUE, constant,
+                           COLUMN_DESCRIPTION, v->con_names[i],
                            -1);
     }
 
@@ -1641,8 +1519,8 @@ create_functions_model()
         gtk_list_store_set(model, &iter,
                            COLUMN_NUMBER, i,
                            COLUMN_EDITABLE, TRUE,
-                           COLUMN_VALUE, g_strdup(v->fun_vals[i]),
-                           COLUMN_DESCRIPTION, g_strdup(v->fun_names[i]),
+                           COLUMN_VALUE, v->fun_vals[i],
+                           COLUMN_DESCRIPTION, v->fun_names[i],
                            -1);
     }
 
@@ -1737,56 +1615,32 @@ request_change_mode()
 static gboolean
 bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event)
 {
-    double number;
-    unsigned long long lval;
-    int n, MP1[MP_SIZE], index;
-    struct exprm_state *e =  get_state();
+    guint64 value;
+    int index;
+    const gchar *text;
+    char buf[MAX_DISPLAY];
+    int MP[MP_SIZE];
+    
+    index = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(event_box), "bit_index"));
+    assert(display_get_unsigned_integer(&v->display, &value));
+    value ^= (1LL << (MAXBITS - index - 1));
+    
+    /* FIXME: Convert to since we don't support setting MP numbers from 64 bit integers */
+    SNPRINTF(buf, MAX_DISPLAY, "%llu", value);
+    MPstr_to_num(buf, DEC, MP);
 
-    index = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(event_box),
-                                              "bit_index"));
-    n = MAXBITS - index - 1;
+    /* FIXME: Set as string as display_set_number doesn't store correctly */
+    make_number(buf, MAX_DISPLAY, MP, v->base, FALSE);
+    display_set_string(&v->display, buf, -1);
+    display_refresh(&v->display);
 
-    switch (v->syntax) {
-        case NPA:
-            MPstr_to_num(v->display, v->base, MP1);
-            break;
-        case EXPRS: {
-            int ret = display_is_usable_number(e->ans);
-            assert(!ret);
-            mpstr(e->ans, MP1);
-        }
-        break;
-        default:
-            assert(FALSE);
-    }
+    text = gtk_label_get_text(GTK_LABEL(X->bits[index]));
+    if (strcmp(text, " 0") == 0)
+        text = " 1";
+    else
+        text = " 0";        
+    gtk_label_set_text(GTK_LABEL(X->bits[index]), text);
 
-    number = mp_cast_to_double(MP1);
-    lval = (long long) number;
-
-    if (lval & (1LL << n)) {
-        lval &= ~(1LL << n);
-        gtk_label_set_text(GTK_LABEL(X->bits[index]), " 0");
-    } else {
-        lval |=  (1LL << n);
-        gtk_label_set_text(GTK_LABEL(X->bits[index]), " 1");
-    }
-    number = (double) lval;
-
-    switch (v->syntax) {
-        case NPA:
-            mp_set_from_double(number, v->MPdisp_val);
-            display_set_number(v->MPdisp_val);
-            break;
-        case EXPRS:
-            mp_set_from_double(number, e->ans);
-            display_set_string("Ans");
-            display_refresh(-1);
-            break;
-        default:
-            assert(FALSE);
-    }
-
-    v->ltr.toclear = 0;
     return (TRUE);
 }
 
@@ -1864,11 +1718,14 @@ get_constant(int n)
  
     SNPRINTF(vkey, MAXLINE, "constant%1dvalue", n);
     if ((vline = get_resource(vkey)) == NULL) {
+        g_free(nline);
         return;
     }   
 
     MPstr_to_num(vline, DEC, v->MPcon_vals[n]);
     STRNCPY(v->con_names[n], nline, MAXLINE - 1);
+    g_free(nline);
+    g_free(vline);
 }
 
 
@@ -1902,16 +1759,19 @@ get_function(int n)
  
     SNPRINTF(nkey, MAXLINE, "function%1dname", n);
     if ((nline = get_resource(nkey)) == NULL) {
-        return;  
+        return;
     }    
  
     SNPRINTF(vkey, MAXLINE, "function%1dvalue", n);
     if ((vline = get_resource(vkey)) == NULL) {
+        g_free(nline);
         return;
     }   
  
     STRNCPY(v->fun_vals[n], convert(vline), MAXLINE - 1);
     STRNCPY(v->fun_names[n], nline, MAXLINE - 1);
+    g_free(nline);
+    g_free(vline);
 }
 
 
@@ -2037,34 +1897,34 @@ kframe_key_press_cb(GtkWidget *widget, GdkEventKey *event)
     if (state == GDK_CONTROL_MASK && v->modetype == SCIENTIFIC) {
         switch (event->keyval) {
             case GDK_0:
-                do_accuracy(0);
+                do_button(KEY_SET_ACCURACY, 0);
                 return (TRUE);
             case GDK_1:
-                do_accuracy(1);
+                do_button(KEY_SET_ACCURACY, 1);
                 return (TRUE);
             case GDK_2:
-                do_accuracy(2);
+                do_button(KEY_SET_ACCURACY, 2);
                 return (TRUE);
             case GDK_3:
-                do_accuracy(3);
+                do_button(KEY_SET_ACCURACY, 3);
                 return (TRUE);
             case GDK_4:
-                do_accuracy(4);
+                do_button(KEY_SET_ACCURACY, 4);
                 return (TRUE);
             case GDK_5:
-                do_accuracy(5);
+                do_button(KEY_SET_ACCURACY, 5);
                 return (TRUE);
             case GDK_6:
-                do_accuracy(6);
+                do_button(KEY_SET_ACCURACY, 6);
                 return (TRUE);
             case GDK_7:
-                do_accuracy(7);
+                do_button(KEY_SET_ACCURACY, 7);
                 return (TRUE);
             case GDK_8:
-                do_accuracy(8);
+                do_button(KEY_SET_ACCURACY, 8);
                 return (TRUE);
             case GDK_9:
-                do_accuracy(9);
+                do_button(KEY_SET_ACCURACY, 9);
                 return (TRUE);
         }
     }
@@ -2135,7 +1995,6 @@ copy_cb(GtkWidget *widget)
 static void
 get_proc(GtkClipboard *clipboard, const gchar *buffer, gpointer data)
 {
-    int ret;
     gchar *dstp, *end_buffer, *srcp, *text, c;
 
     if (buffer == NULL) {
@@ -2203,25 +2062,10 @@ get_proc(GtkClipboard *clipboard, const gchar *buffer, gpointer data)
     }
     *dstp++ = '\0';
 
-    switch (v->syntax) {
-        case NPA:
-            ret = lr_parse((char *) text, v->MPdisp_val);
-            if (!ret) {
-                display_set_number(v->MPdisp_val);
-            } else {
-                ui_set_statusbar(_("Clipboard contained malformed calculation"),
-                                 "gtk-dialog-error");
-            }
-            break;
-    
-        case EXPRS:
-            display_insert((char *) text, get_cursor()); // FIXME: Move out of gtk.c
-            display_refresh(-1);
-            break;
-    
-        default:
-            assert(0);
-    }
+    display_set_cursor(&v->display, get_cursor()); // FIXME: Move out of gtk.c
+    display_insert(&v->display, (char *) text);
+    display_set_cursor(&v->display, -1);
+    display_refresh(&v->display);
     free(text);
 }
 
@@ -2260,7 +2104,7 @@ popup_paste_cb(GtkWidget *menu)
 static void
 undo_cb(GtkWidget *widget)
 {
-    perform_undo();
+    do_button(KEY_UNDO, 0);
 }
 
 
@@ -2268,8 +2112,9 @@ undo_cb(GtkWidget *widget)
 static void
 redo_cb(GtkWidget *widget)
 {
-    perform_redo();
-    display_refresh(-1);
+    do_button(KEY_REDO, 0);    
+    display_set_cursor(&v->display, -1);
+    display_refresh(&v->display);
 }
 
 
@@ -2342,15 +2187,6 @@ show_registers_cb(GtkWidget *widget)
 }
 
 
-static void
-arithmetic_mode_cb(GtkWidget *widget)
-{
-    enum syntax mode;
-    mode = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget)) ? EXPRS : NPA;
-    ui_set_syntax_mode(mode);
-}
-
-
 /*ARGSUSED*/
 static void
 mode_radio_cb(GtkWidget *menu)
@@ -2376,7 +2212,7 @@ mode_radio_cb(GtkWidget *menu)
  *
  * (unless we are in Scientific mode with Decimal numeric base and Fixed).
  */
-    if (display_is_result() &&
+    if (display_is_result(&v->display) &&
         ((v->modetype != SCIENTIFIC) ||
          (v->dtype == FIX && v->base == DEC))) {
         v->modetype = mode;
@@ -2398,7 +2234,7 @@ accuracy_radio_cb(GtkWidget *widget)
     int count;
     count = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "accuracy"));
     if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) {
-        do_accuracy(count);
+        do_button(KEY_SET_ACCURACY, count);
     }
 }
 
@@ -2419,7 +2255,7 @@ accuracy_other_cb(GtkWidget *widget)
 static void
 accuracy_default_cb(GtkWidget *widget)
 {
-    do_accuracy(DEFAULT_ACCURACY);
+    do_button(KEY_SET_ACCURACY, DEFAULT_ACCURACY);
 }
 
 
@@ -2449,7 +2285,7 @@ spframe_response_cb(GtkWidget *dialog, gint response_id)
     int val;
     if (response_id == GTK_RESPONSE_OK) {
         val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(X->precision_spin));
-        ui_set_accuracy(val);
+        do_button(KEY_SET_ACCURACY, val);
     }
     
     gtk_widget_hide(dialog);
@@ -2590,7 +2426,6 @@ create_kframe()
     CONNECT_SIGNAL(store_menu_cb);
     CONNECT_SIGNAL(recall_menu_cb);
     CONNECT_SIGNAL(exchange_menu_cb);
-    CONNECT_SIGNAL(arithmetic_mode_cb);
     CONNECT_SIGNAL(mouse_button_cb);
     /* Detect when populating the right-click menu to enable pasting */
     CONNECT_SIGNAL(buffer_populate_popup_cb);
@@ -2743,8 +2578,6 @@ create_kframe()
     set_menubar_tooltip("show_thousands_separator_menu");
     set_menubar_tooltip("show_bitcalculating_menu");
     set_menubar_tooltip("show_registers_menu");
-    set_menubar_tooltip("ltr_precedence_menu");
-    set_menubar_tooltip("arithmetic_precedence_menu");
     set_menubar_tooltip("help_menu");
     set_menubar_tooltip("about_menu");
 
@@ -2874,6 +2707,8 @@ read_cfdefs()        /* Read constant/function definitions. */
 void
 ui_init(int *argc, char ***argv)
 {  
+    gchar *path;
+    
     X = (XVars) LINT_CAST(calloc(1, sizeof(struct Xobject)));
         
     gtk_init(argc, argv);
@@ -2883,9 +2718,11 @@ ui_init(int *argc, char ***argv)
     gtk_rc_get_default_files();
 
     v->home = (char *) g_get_home_dir();
-    gtk_rc_parse(g_build_path(v->home, RCNAME, NULL));
+    path = g_build_path(v->home, RCNAME, NULL);
+    gtk_rc_parse(path);
+    g_free(path);
 
-    gtk_window_set_default_icon_name("gnome-calculator");
+    gtk_window_set_default_icon_name("accessories-calculator");
 }
 
 
@@ -2904,13 +2741,17 @@ ui_load(void)
     
     /* Load configuration */
     resource = get_resource(R_BITCALC);
-    show_bit = resource != NULL && strcmp(resource, Rcstr[0]) != 0;
+    if(resource) {
+        show_bit = strcmp(resource, Rcstr[0]) != 0;
+        g_free(resource);
+    } else {
+        show_bit = FALSE;
+    }
 
     ui_set_show_thousands_separator(v->show_tsep);
     ui_set_show_trailing_zeroes(v->show_zeroes);
     ui_set_show_bitcalculating(show_bit);
     
-    ui_set_syntax_mode(v->syntax);
     ui_set_mode(v->modetype);
     ui_set_numeric_mode(FIX);
     ui_set_base(v->base);
@@ -2949,9 +2790,6 @@ ui_start(void)
 
     gtk_widget_show(X->kframe);
 
-    /* Init expression mode.
-     * This must be executed after do_base is called at init.
-     */    
     reset_display();
 
     gtk_main();
